use strict;
use Data::Dumper;
use DE_EPAGES::ClickAndBuy::API::Soap::CaBWebService;
use DE_EPAGES::ClickAndBuy::API::XML::Parser;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Object::API::Language qw ( GetPKeyLanguageByCode );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use Readonly;
use Test::More;
## no critic (ProhibitPathConcatenation ProhibitMagicNumbers ProhibitLongLines)
Readonly my $ROOTPATH => '/TestStore';
use feature 'say';
sub Main {
    RunOnStore(
        'Store' => 'Store',
        'Sub'   => sub {

            # startUp
            ok( Init(), 'initialization' );

            # tests
            eval {
                    local *DateTime::strftime = sub{
                        return '20130130105058';
                    };

                    local $DE_EPAGES::Object::API::Language::LANGUAGEID = GetPKeyLanguageByCode( 'en' );

                    my $PaymentMethod = LoadObjectByPath($ROOTPATH.'/TestShop/PaymentMethods/ClickAndBuyNew');

                    note('Creating Parser and the Web Service Client');

                    my $webClient = DE_EPAGES::ClickAndBuy::API::Soap::CaBWebService->new($PaymentMethod->get('IsInTestMode'));

                    $webClient->{'TransportHook'} = \&createMerchantSuccessServer;
                    my $response = $webClient->createMerchantRegistration($PaymentMethod);

                    my $expected = {
                      'registrationSharedSecret' => 'CF03290D2F502DEEA45D4F2C8C220A17',
                      'mmsSharedSecret' => 'T5Uq8GG~',
                      'projectSharedSecret' => 'E2483A3D0019023104D17E4ED39D7120',
                      'registrationURL' => 'https://merchant.clickandbuy-s1.com/frontend/secure/merchant/apiRegistration?mid=1187860001&t=MjAxMzAxMjgwODQ0NTE6OjlFMjkwRkRCQjIzM0ZDMzAxMTE3MjY3NDc3NDk5QUZFRkE2MDRCMzM',
                      'projectID' => bless( {
                                              'value' => [
                                                           '187863001',
                                                           '1'
                                                         ],
                                              'sign' => '+'
                                            }, 'Math::BigInt' ),
                      'status' => 'CREATED',
                      'merchantID' => bless( {
                                               'value' => [
                                                            '187860001',
                                                            '1'
                                                          ],
                                               'sign' => '+'
                                             }, 'Math::BigInt' )
                    };
                    is_deeply($response, $expected, 'Create Merchant Response');

                    $webClient->{'TransportHook'} = \&getMerchantRegistrationStatusServer;
                    $response = $webClient->getMerchantRegistrationStatus($PaymentMethod);

                    is_deeply($response, $expected, 'Get Merchant Status Response');

                    my $Order = LoadObjectByPath($ROOTPATH.'/TestShop/Customers/1001/Orders/1005');## no critic (ProhibitPathConcatenation)

                    $webClient->{'TransportHook'} = \&payRequestServer;
                    $response = $webClient->payRequest($Order);

                    $expected = {
                      'recurringPaymentAuthorizationID' => undef,
                      'transactionID' => bless( {
                                                  'value' => [
                                                               '837653001',
                                                               '1'
                                                             ],
                                                  'sign' => '+'
                                                }, 'Math::BigInt' ),
                      'crn' => undef,
                      'transactionStatus' => 'CREATED',
                      'redirectURL' => 'https://checkout.clickandbuy-s1.com/checkout/?tx=1837653001&s=6C31AF03FD32AE9D&h=1ED276197A9E70DEB43A3194B5C08C5B67D1F705'
                    };

                    is_deeply($response, $expected, 'Pay Request Response');

                    $webClient->{'TransportHook'} = \&statusRequestServer;
                    $response = $webClient->statusRequest($Order);

                    $expected = {
                      '50FFE7A2-8FF8-9E24-95C6-AC1532012695' => {
                                                                'transactionID' => bless( {
                                                                                            'value' => [
                                                                                                         '840749001',
                                                                                                         1
                                                                                                       ],
                                                                                            'sign' => '+'
                                                                                          }, 'Math::BigInt' ),
                                                                'transactionStatus' => 'ABORTED'
                                                              },
                      '50FFE7A2-8FF8-9E24-95C6-AC1532012692' => {
                                                                'transactionID' => bless( {
                                                                                            'value' => [
                                                                                                         '840749001',
                                                                                                         1
                                                                                                       ],
                                                                                            'sign' => '+'
                                                                                          }, 'Math::BigInt' ),
                                                                'transactionStatus' => 'ABORTED'
                                                              }
                    };

                    is_deeply($response, $expected, 'Status Request Response');

                    $Order = LoadObjectByPath($ROOTPATH.'/TestShop/Customers/1001/Orders/1006');## no critic (ProhibitPathConcatenation)
                    $webClient->{'TransportHook'} = \&refundRequestServer;
                    $response = $webClient->refundRequest($Order, 215.95);


                    $expected = {
                      'recurringPaymentAuthorizationID' => undef,
                      'transactionID' => '147852369',
                      'crn' => undef,
                      'transactionStatus' => 'SUCCESS',
                      'redirectURL' => undef
                    };

                    is_deeply($response, $expected, 'Refund Request Response');

                    $webClient->{'TransportHook'} = \&cancelRequestServer;
                    $response = $webClient->cancelRequest($Order);

                    $expected = {
                      'recurringPaymentAuthorizationID' => undef,
                      'transactionID' => '1889116001',
                      'crn' => '128187238',
                      'transactionStatus' => 'CANCELLED',
                      'redirectURL' => undef
                    };

                    is_deeply($response, $expected, 'Cancel Request Response');

                 };

            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }
            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}


sub getMerchantRegistrationStatusServer($$) {
    my ($request, $trace) = @_;

    my $Shop = LoadObjectByPath($ROOTPATH.'/TestShop');
    my $UrlSf = $Shop->get('WebServerScriptUrlSSL');
    my $content = $request->decoded_content;
    my $expectedRequest = <<__EXPECTED_CONTENT;
<?xml version="1.0" encoding="UTF-8"?>
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"><SOAP-ENV:Body><cab:getMerchantRegistrationStatus_Request xmlns:cab="http://api.clickandbuy.com/webservices/pay_1_1_0/"><cab:details><cab:businessOriginID>ePages</cab:businessOriginID><cab:merchantID>1185654001</cab:merchantID><cab:token>20130130105058::C101DD8C9942D91354964CDE16ECD589B98CB84D</cab:token></cab:details></cab:getMerchantRegistrationStatus_Request></SOAP-ENV:Body></SOAP-ENV:Envelope>
__EXPECTED_CONTENT

    is($content, $expectedRequest , 'Get Merchant Status Request');
    HTTP::Response->new(200, 'Constant'
      , [ 'Content-Type' => 'text/xml' ]
      , <<__ANSWER)
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"><SOAP-ENV:Header/><SOAP-ENV:Body><ns2:getMerchantRegistrationStatus_Response xmlns:ns2="http://api.clickandbuy.com/webservices/pay_1_1_0/"><ns2:requestTrackingID>3b28235f#m1-s1#20130128084430</ns2:requestTrackingID><ns2:registrationInfo><ns2:merchantID>1187860001</ns2:merchantID><ns2:registrationStatus>CREATED</ns2:registrationStatus><ns2:registrationSharedSecret>CF03290D2F502DEEA45D4F2C8C220A17</ns2:registrationSharedSecret><ns2:registrationURL>https://merchant.clickandbuy-s1.com/frontend/secure/merchant/apiRegistration?mid=1187860001&amp;t=MjAxMzAxMjgwODQ0NTE6OjlFMjkwRkRCQjIzM0ZDMzAxMTE3MjY3NDc3NDk5QUZFRkE2MDRCMzM</ns2:registrationURL></ns2:registrationInfo><ns2:integrationInfoList><ns2:integrationInfo><ns2:settlementInfo><ns2:settlementAccountID>1187862001</ns2:settlementAccountID><ns2:settlementAccountCurrency>EUR</ns2:settlementAccountCurrency><ns2:settlementAccountName>myFirstSettlementAccount</ns2:settlementAccountName></ns2:settlementInfo><ns2:feeCardInfo><ns2:invoicingCycle>14</ns2:invoicingCycle><ns2:settlementDelay>5</ns2:settlementDelay><ns2:averageTicketSize><ns2:amount>50.00</ns2:amount><ns2:currency>EUR</ns2:currency></ns2:averageTicketSize></ns2:feeCardInfo><ns2:projectInfo><ns2:projectID>1187863001</ns2:projectID><ns2:projectName>Milestones4444</ns2:projectName><ns2:projectSharedSecret>E2483A3D0019023104D17E4ED39D7120</ns2:projectSharedSecret><ns2:mmsURL>$UrlSf/?ChangeAction=BlockMonitor&amp;ChangeAction=ClickAndBuyMMS&amp;ObjectPath=/Shops/DemoShop</ns2:mmsURL><ns2:mmsSharedSecret>T5Uq8GG~</ns2:mmsSharedSecret></ns2:projectInfo></ns2:integrationInfo></ns2:integrationInfoList></ns2:getMerchantRegistrationStatus_Response></SOAP-ENV:Body></SOAP-ENV:Envelope>
__ANSWER

}

sub createMerchantSuccessServer($$) {
    my $Shop = LoadObjectByPath($ROOTPATH.'/TestShop');
    my $UrlAdmin = $Shop->get('WebServerScriptAdminUrlSSL');
    my $UrlSf = $Shop->get('WebServerScriptUrlSSL');
    my ($request, $trace) = @_;
    my $content = $request->decoded_content;
    my $expectedRequest = <<__EXPECTED_CONTENT;
<?xml version="1.0" encoding="UTF-8"?>
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"><SOAP-ENV:Body><cab:createMerchantRegistration_Request xmlns:cab="http://api.clickandbuy.com/webservices/pay_1_1_0/"><cab:details><cab:registrationData><cab:businessOriginID>ePages</cab:businessOriginID><cab:returnURL>$UrlAdmin/?ChangeAction=BlockMonitor&amp;ChangeAction=ClickAndBuyRegisterCallback&amp;ObjectPath=/TestStore/TestShop/PaymentMethods/ClickAndBuyNew&amp;ViewAction=MBO-ViewSettings</cab:returnURL></cab:registrationData><cab:merchantData><cab:companyName>TestCompany</cab:companyName><cab:vatID>123456789</cab:vatID><cab:companyAddress><cab:street>TestStreet</cab:street><cab:zip>12345</cab:zip><cab:city>TestCity</cab:city><cab:country>DE</cab:country></cab:companyAddress><cab:emailAddress>test\@email.com</cab:emailAddress><cab:website>$UrlSf/?ObjectPath=/TestStore/TestShop</cab:website><cab:language>en</cab:language><cab:adminFirstName>TestFN</cab:adminFirstName><cab:adminLastName>TestLN</cab:adminLastName></cab:merchantData><cab:integrationData><cab:projectData><cab:name>TestShop</cab:name><cab:mmsURL>$UrlSf/?ChangeAction=BlockMonitor&amp;ChangeAction=ClickAndBuyMMS&amp;ObjectPath=/TestStore/TestShop</cab:mmsURL></cab:projectData></cab:integrationData></cab:details></cab:createMerchantRegistration_Request></SOAP-ENV:Body></SOAP-ENV:Envelope>
__EXPECTED_CONTENT

    is($content, $expectedRequest , 'Create Merchant Request');
    HTTP::Response->new(200, 'Constant'
      , [ 'Content-Type' => 'text/xml' ]
      , <<__ANSWER)
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"><SOAP-ENV:Header/><SOAP-ENV:Body><ns2:createMerchantRegistration_Response xmlns:ns2="http://api.clickandbuy.com/webservices/pay_1_1_0/"><ns2:requestTrackingID>3b28235f#m1-s1#20130128084430</ns2:requestTrackingID><ns2:registrationInfo><ns2:merchantID>1187860001</ns2:merchantID><ns2:registrationStatus>CREATED</ns2:registrationStatus><ns2:registrationSharedSecret>CF03290D2F502DEEA45D4F2C8C220A17</ns2:registrationSharedSecret><ns2:registrationURL>https://merchant.clickandbuy-s1.com/frontend/secure/merchant/apiRegistration?mid=1187860001&amp;t=MjAxMzAxMjgwODQ0NTE6OjlFMjkwRkRCQjIzM0ZDMzAxMTE3MjY3NDc3NDk5QUZFRkE2MDRCMzM</ns2:registrationURL></ns2:registrationInfo><ns2:integrationInfoList><ns2:integrationInfo><ns2:settlementInfo><ns2:settlementAccountID>1187862001</ns2:settlementAccountID><ns2:settlementAccountCurrency>EUR</ns2:settlementAccountCurrency><ns2:settlementAccountName>myFirstSettlementAccount</ns2:settlementAccountName></ns2:settlementInfo><ns2:feeCardInfo><ns2:invoicingCycle>14</ns2:invoicingCycle><ns2:settlementDelay>5</ns2:settlementDelay><ns2:averageTicketSize><ns2:amount>50.00</ns2:amount><ns2:currency>EUR</ns2:currency></ns2:averageTicketSize></ns2:feeCardInfo><ns2:projectInfo><ns2:projectID>1187863001</ns2:projectID><ns2:projectName>Milestones4444</ns2:projectName><ns2:projectSharedSecret>E2483A3D0019023104D17E4ED39D7120</ns2:projectSharedSecret><ns2:mmsURL>$UrlSf/?ChangeAction=BlockMonitor&amp;ChangeAction=ClickAndBuyMMS&amp;ObjectPath=/Shops/DemoShop</ns2:mmsURL><ns2:mmsSharedSecret>T5Uq8GG~</ns2:mmsSharedSecret></ns2:projectInfo></ns2:integrationInfo></ns2:integrationInfoList></ns2:createMerchantRegistration_Response></SOAP-ENV:Body></SOAP-ENV:Envelope>
__ANSWER

}

sub payRequestServer($$) {
    my $Shop = LoadObjectByPath($ROOTPATH.'/TestShop');
    my $UrlSf = $Shop->get('WebServerScriptUrlSSL');
    my ($request, $trace) = @_;
    my $content = $request->decoded_content;
    my $expectedRequest = <<__EXPECTED_CONTENT;
<?xml version="1.0" encoding="UTF-8"?>
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"><SOAP-ENV:Body><cab:payRequest_Request xmlns:cab="http://api.clickandbuy.com/webservices/pay_1_1_0/"><cab:authentication><cab:merchantID>1185654001</cab:merchantID><cab:projectID>1185667001</cab:projectID><cab:token>20130130105058::989C4AE760A43F05245A7BF9073FABF45AB8F387</cab:token></cab:authentication><cab:details><cab:amount><cab:amount>215.95</cab:amount><cab:currency>EUR</cab:currency></cab:amount><cab:orderDetails><cab:text>Your order at #Shop.NameOrAlias</cab:text><cab:itemList><cab:item><cab:itemType>ITEM</cab:itemType><cab:description>Jack Wolfskin Blizzard Jacket</cab:description><cab:quantity>1</cab:quantity><cab:unitPrice><cab:amount>215.95</cab:amount><cab:currency>EUR</cab:currency></cab:unitPrice><cab:totalPrice><cab:amount>215.95</cab:amount><cab:currency>EUR</cab:currency></cab:totalPrice></cab:item><cab:item><cab:itemType>ITEM</cab:itemType><cab:description>ClickandBuy</cab:description><cab:quantity>1</cab:quantity><cab:unitPrice><cab:amount>1</cab:amount><cab:currency>EUR</cab:currency></cab:unitPrice><cab:totalPrice><cab:amount>1</cab:amount><cab:currency>EUR</cab:currency></cab:totalPrice></cab:item><cab:item><cab:itemType>TOTAL</cab:itemType><cab:description> </cab:description><cab:totalPrice><cab:amount>215.95</cab:amount><cab:currency>EUR</cab:currency></cab:totalPrice></cab:item></cab:itemList></cab:orderDetails><cab:billing><cab:consumer><cab:firstName>Max</cab:firstName><cab:lastName>Mustermann</cab:lastName><cab:address><cab:street>Blumenweg</cab:street><cab:houseNumber>2</cab:houseNumber><cab:zip>09876</cab:zip><cab:city>Neustadt</cab:city><cab:country>DE</cab:country></cab:address></cab:consumer></cab:billing><cab:shipping><cab:consumer><cab:firstName>Max</cab:firstName><cab:lastName>Mustermann</cab:lastName><cab:address><cab:street>Blumenweg</cab:street><cab:houseNumber>2</cab:houseNumber><cab:zip>09876</cab:zip><cab:city>Neustadt</cab:city><cab:country>DE</cab:country></cab:address></cab:consumer></cab:shipping><cab:successExpiration>1440</cab:successExpiration><cab:successURL>$UrlSf/?ChangeAction=CheckClickAndBuyResult&amp;LIGUID=50FFE7A2-8FF8-9E24-95C6-AC1532012695&amp;ObjectPath=/TestStore/TestShop&amp;Success=1</cab:successURL><cab:failureURL>$UrlSf/?ChangeAction=CheckClickAndBuyResult&amp;LIGUID=50FFE7A2-8FF8-9E24-95C6-AC1532012695&amp;ObjectPath=/TestStore/TestShop</cab:failureURL><cab:externalID>50FFE7A2-8FF8-9E24-95C6-AC1532012695</cab:externalID><cab:consumerLanguage>en</cab:consumerLanguage></cab:details></cab:payRequest_Request></SOAP-ENV:Body></SOAP-ENV:Envelope>
__EXPECTED_CONTENT
    is($content, $expectedRequest , 'Pay Request');
    HTTP::Response->new(200, 'Constant'
      , [ 'Content-Type' => 'text/xml' ]
      , <<__ANSWER)
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"><SOAP-ENV:Header/><SOAP-ENV:Body><ns2:payRequest_Response xmlns:ns2="http://api.clickandbuy.com/webservices/pay_1_1_0/"><ns2:requestTrackingID>3b272ac2#m1-s1#20130130110755</ns2:requestTrackingID><ns2:transaction><ns2:transactionID>1837653001</ns2:transactionID><ns2:externalID>5108DE45-1DF1-5E9F-BB4C-AC153201E337</ns2:externalID><ns2:transactionStatus>CREATED</ns2:transactionStatus><ns2:transactionType>PAY</ns2:transactionType><ns2:redirectURL>https://checkout.clickandbuy-s1.com/checkout/?tx=1837653001&amp;s=6C31AF03FD32AE9D&amp;h=1ED276197A9E70DEB43A3194B5C08C5B67D1F705</ns2:redirectURL></ns2:transaction></ns2:payRequest_Response></SOAP-ENV:Body></SOAP-ENV:Envelope>
__ANSWER
}

sub statusRequestServer($$) {
    my ($request, $trace) = @_;
    my $content = $request->decoded_content;
    my $expectedRequest = <<__EXPECTED_CONTENT;
<?xml version="1.0" encoding="UTF-8"?>
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"><SOAP-ENV:Body><cab:statusRequest_Request xmlns:cab="http://api.clickandbuy.com/webservices/pay_1_1_0/"><cab:authentication><cab:merchantID>1185654001</cab:merchantID><cab:projectID>1185667001</cab:projectID><cab:token>20130130105058::989C4AE760A43F05245A7BF9073FABF45AB8F387</cab:token></cab:authentication><cab:details><cab:externalIDList><cab:externalID>50FFE7A2-8FF8-9E24-95C6-AC1532012695</cab:externalID></cab:externalIDList></cab:details></cab:statusRequest_Request></SOAP-ENV:Body></SOAP-ENV:Envelope>
__EXPECTED_CONTENT
    is($content, $expectedRequest , 'Status Request');
    HTTP::Response->new(200, 'Constant'
      , [ 'Content-Type' => 'text/xml' ]
      , <<__ANSWER)
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"><SOAP-ENV:Header/><SOAP-ENV:Body><ns2:statusRequest_Response xmlns:ns2="http://api.clickandbuy.com/webservices/pay_1_1_0/"><ns2:requestTrackingID>3b282e9b#m1-s1#20130131161734</ns2:requestTrackingID><ns2:transactionList><ns2:transaction><ns2:transactionID>1840749001</ns2:transactionID><ns2:externalID>50FFE7A2-8FF8-9E24-95C6-AC1532012695</ns2:externalID><ns2:transactionStatus>ABORTED</ns2:transactionStatus><ns2:transactionType>PAY</ns2:transactionType></ns2:transaction><ns2:transaction><ns2:transactionID>1840749001</ns2:transactionID><ns2:externalID>50FFE7A2-8FF8-9E24-95C6-AC1532012692</ns2:externalID><ns2:transactionStatus>ABORTED</ns2:transactionStatus><ns2:transactionType>PAY</ns2:transactionType></ns2:transaction></ns2:transactionList></ns2:statusRequest_Response></SOAP-ENV:Body></SOAP-ENV:Envelope>
__ANSWER
}

sub refundRequestServer($$) {
    my ($request, $trace) = @_;
    my $content = $request->decoded_content;
    my $expectedRequest = <<__EXPECTED_CONTENT;
<?xml version="1.0" encoding="UTF-8"?>
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"><SOAP-ENV:Body><cab:refundRequest_Request xmlns:cab="http://api.clickandbuy.com/webservices/pay_1_1_0/"><cab:authentication><cab:merchantID>1185654001</cab:merchantID><cab:projectID>1185667001</cab:projectID><cab:token>20130130105058::989C4AE760A43F05245A7BF9073FABF45AB8F387</cab:token></cab:authentication><cab:details><cab:amount><cab:amount>215.95</cab:amount><cab:currency>EUR</cab:currency></cab:amount><cab:transactionID>147852369</cab:transactionID></cab:details></cab:refundRequest_Request></SOAP-ENV:Body></SOAP-ENV:Envelope>
__EXPECTED_CONTENT
    is($content, $expectedRequest , 'Refund Request');
    HTTP::Response->new(200, 'Constant'
      , [ 'Content-Type' => 'text/xml' ]
      , <<__ANSWER)
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"><SOAP-ENV:Header/><SOAP-ENV:Body><ns2:refundRequest_Response xmlns:ns2="http://api.clickandbuy.com/webservices/pay_1_1_0/"><ns2:requestTrackingID>3b272ac2#m1-s1#20130130110755</ns2:requestTrackingID><ns2:transaction><ns2:transactionID>147852369</ns2:transactionID><ns2:externalID>50FFE7A2-8FF8-9E24-95C6-AC1532012147</ns2:externalID><ns2:transactionStatus>SUCCESS</ns2:transactionStatus><ns2:transactionType>REFUND</ns2:transactionType></ns2:transaction></ns2:refundRequest_Response></SOAP-ENV:Body></SOAP-ENV:Envelope>
__ANSWER
}

sub cancelRequestServer($$) {
    my ($request, $trace) = @_;
    my $content = $request->decoded_content;
    my $expectedRequest = <<__EXPECTED_CONTENT;
<?xml version="1.0" encoding="UTF-8"?>
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"><SOAP-ENV:Body><cab:cancelRequest_Request xmlns:cab="http://api.clickandbuy.com/webservices/pay_1_1_0/"><cab:authentication><cab:merchantID>1185654001</cab:merchantID><cab:projectID>1185667001</cab:projectID><cab:token>20130130105058::989C4AE760A43F05245A7BF9073FABF45AB8F387</cab:token></cab:authentication><cab:details><cab:cancelMode>TX</cab:cancelMode><cab:cancelIdentifier><cab:transactionID>147852369</cab:transactionID></cab:cancelIdentifier></cab:details></cab:cancelRequest_Request></SOAP-ENV:Body></SOAP-ENV:Envelope>
__EXPECTED_CONTENT
    is($content, $expectedRequest , 'Cancel Request');
    HTTP::Response->new(200, 'Constant'
      , [ 'Content-Type' => 'text/xml' ]
      , <<__ANSWER)
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"><SOAP-ENV:Header/><SOAP-ENV:Body><ns2:cancelRequest_Response xmlns:ns2="http://api.clickandbuy.com/webservices/pay_1_1_0/"><ns2:requestTrackingID>3b2f123c#m1-s1#20130218153956</ns2:requestTrackingID><ns2:transaction><ns2:transactionID>1889116001</ns2:transactionID><ns2:externalID>5122276F-6F1A-D611-4E13-D5809AA56304</ns2:externalID><ns2:crn>128187238</ns2:crn><ns2:transactionStatus>CANCELLED</ns2:transactionStatus><ns2:transactionType>PAY</ns2:transactionType></ns2:transaction></ns2:cancelRequest_Response></SOAP-ENV:Body></SOAP-ENV:Envelope>
__ANSWER
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'TestStore' } );
    ImportObjects( 'ImportOrder.xml', $ROOTPATH );
    return 1;
}

RunScript( Sub => \&Main );

## use critic (ProhibitPathConcatenation ProhibitMagicNumbers ProhibitLongLines)

1;