use strict;
use warnings;
use Test::Deep;
use Test::More tests => 17;

use DE_EPAGES::Core::API::File qw ( DeleteFile );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw ( ExistsObjectByPath JoinObjectPath LoadObjectByPath LoadRootObject );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::PriceList::API::CSV::PriceListExportImport;

use Carp qw ( croak );
use List::Util qw ( first );
use Text::CSV_XS;

use Readonly;
Readonly::Scalar my $ROOTPATH         => 'Test';
Readonly::Scalar my $SHOP             => 'TestShop';
Readonly::Scalar my $CSV_FILENAME     => 'discounts.csv';
Readonly::Scalar my $TMP_FILENAME     => 'tmp.csv';
Readonly::Scalar my $NEW_DISCOUNT     => 100;
Readonly::Hash   my %hExpectedResults => (
    # product alias       => expected product path after CSV import
    'be_4/04/01'          => '"be_4/04/01"',
    'Shirt'               => 'Shirt',
    'be_404/0104/004'     => '"be_4/04/01"/SubProducts/"be_404/0104/004"',
    'be_4040104005'       => '"be_4/04/01"/SubProducts/be_4040104005',
    'cg_0100//50/40/ /01' => '"cg_0100//50/40/ /01"',
    'FancyShirt / 1'      => 'FancyShirt/SubProducts/"FancyShirt / 1"',
    'mlaut hirt'        => 'FancyShirt/SubProducts/"mlaut hirt"',
);
Readonly::Array my @aExpectedPriceLists => (
    'RegularCustomer_EUR',
    '"Regular/PriceList"',
    "\"Li\x{df}t with \x{dc}ml\x{e4}uts\"",
);

sub TestExportImport {
    my $Shop = LoadObjectByPath( JoinObjectPath([ $ROOTPATH, $SHOP ]) );
    my $ProductsPath = join('/', $ROOTPATH, $SHOP, 'Products');
    my $InputTaxModel = $Shop->get('InputTaxModel');

    # ExporterImporter options
    my $ExportImporter = DE_EPAGES::PriceList::API::CSV::PriceListExportImport->new;
    my $hFileFormat = {
        'LanguageCode2s' => [],
    };

    my $PriceLists = $Shop->get('PriceLists');
    foreach my $Alias ( keys %hExpectedResults ) {
        # csvExport using PriceListExportDriver
        my $ProductAlias = $hExpectedResults{$Alias};
        my $Product = LoadObjectByPath("$ProductsPath/$ProductAlias");  ## no critic qw(ProhibitPathConcatenation)

        SKIP: {
            skip ('Product has missing attribute HasSubOwnPrices set to \'1\'.)', 2)
                if ( $Product->get('SuperProduct') && !$Product->get('HasSubOwnPrices') );

            $hFileFormat->{'LocaleID'} = $Shop->get('LocaleID');  # LocaleID is deleted during csvExport
            $ExportImporter->csvExport($Shop, $CSV_FILENAME, $hFileFormat, [ $Product->id ]);

            # csvImport using an own handler (cf. below)
            $hFileFormat->{'LocaleID'} = $Shop->get('LocaleID');  # LocaleID is deleted during csvExport
            my $Importer = DE_EPAGES::PriceList::t::MyImportHandler->importCSV($CSV_FILENAME, $hFileFormat);
            is( $Importer->{'Items'}->[0]->{'Product'}, $hExpectedResults{$Alias}, "test '$Alias'");

            # modify some CSV values
            my $CSV = Text::CSV_XS->new({ $Importer->converter->formatSpec })
                or croak('Cannot use CSV: ' . Text::CSV->error_diag ());

            my $IsFirstLine = 1;
            open (my $CSVData, '<', $CSV_FILENAME) or croak "Could not open '$CSV_FILENAME' $!\n";
            open (my $NewData, '>', $TMP_FILENAME) or croak "Could not open '$TMP_FILENAME' $!\n";
            while ( my $Line = <$CSVData> ) {
                chomp $Line;
                if ( $CSV->parse($Line) ) {
                    my @Fields = $CSV->fields();
                    if ($IsFirstLine == 1) {
                        $CSV->print($NewData, \@Fields);
                        $IsFirstLine = 0;
                        next;
                    }
                    $Fields[4] = $NEW_DISCOUNT;
                    $CSV->print($NewData, \@Fields);
                } else {
                    warn "Line could not be parsed: $Line\n";
                }
            }
            close ($CSVData) or warn $! ? "Error closing $CSVData: $!" : "Exit status $?";
            close ($NewData) or warn $! ? "Error closing $NewData: $!" : "Exit status $?";

            # csvImport into database
            $hFileFormat->{'LocaleID'} = $Shop->get('LocaleID');  # LocaleID is delete during csvExport
            $ExportImporter->csvImport($Shop, $TMP_FILENAME, $hFileFormat);

            # verify new discounts set to $NEW_DISCOUNT
            my $Index = first { $PriceLists->[$_]->alias eq 'RegularCustomer_EUR' } 0 .. (scalar @{ $PriceLists });
            my $aQuantityDiscounts = $PriceLists->[$Index]->getQuantityDiscounts( $Product, $InputTaxModel, 1 );
            is( $aQuantityDiscounts->[$Index]->{'Discount'}, $NEW_DISCOUNT, "new discount value for '$Alias' is set" );
        }
    }

    # Cleanup
    DeleteFile($CSV_FILENAME);
    DeleteFile($TMP_FILENAME);

    return;
}

sub TestExportImportOfPriceLists {
    my $Shop = LoadObjectByPath( JoinObjectPath([ $ROOTPATH, $SHOP ]) );

    my $ProductPath = join('/', $ROOTPATH, $SHOP, 'Products', 'Shirt');
    my $Product = LoadObjectByPath($ProductPath);

    my $hFileFormat = {
        'LanguageCode2s' => [],
        'LocaleID'       => $Shop->get('LocaleID'),
    };
    # export discounts for 'Shirt'
    my $ExportImporter = DE_EPAGES::PriceList::API::CSV::PriceListExportImport->new;
    $ExportImporter->csvExport($Shop, $CSV_FILENAME, $hFileFormat, [ $Product->id ]);

    # import discounts for 'Shirt'
    $hFileFormat->{'LocaleID'} = $Shop->get('LocaleID');  # LocaleID is delete during csvExport
    my $Importer = DE_EPAGES::PriceList::t::MyImportHandler->importCSV($CSV_FILENAME, $hFileFormat);

    # compare imported price lists with expected price lists
    my @ImportedPriceLists = map { $_->{'PriceList'} }@{ $Importer->{'Items'} };
    cmp_bag( \@ImportedPriceLists, \@aExpectedPriceLists,
        'all pricelists were imported successfully for \'' . $Product->alias . '\'');
    return;
}

sub Cleanup {
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);

    return 1;
}

sub Init {
    Cleanup();

    my $System = LoadRootObject();
    $System->insertChild('Object', {
        Alias => 'Test'
    });
    ImportObjects('import.xml', $ROOTPATH);

    return 1;
}

RunScript(Debug => 1, Sub => sub {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    TestExportImportOfPriceLists();
    TestExportImport();

    ok( Cleanup(), 'cleanup');
    DisconnectSite();
});


#========================================================================================
# package      DE_EPAGES::PriceList::t::MyImportHandler;
# base         DE_EPAGES::Core::API::CSV::BaseImportHandler
# state        public
#----------------------------------------------------------------------------------------
# description  This module implements a simple handler to read CSV files.
#========================================================================================
package DE_EPAGES::PriceList::t::MyImportHandler;
use base qw ( DE_EPAGES::Core::API::CSV::BaseImportHandler );

sub new {  ## no critic qw(RequireArgUnpacking)
    my $class = shift;
    my $self = $class->SUPER::new(@_);

    $self->{'Items'} = [];

    return $self;
}

sub importLine {
    my $self = shift;

    my @Header = map { /\[(.*?)\]/ } @{ $self->header };
    my %hColumns;
    @hColumns{ @Header } = @{ $self->topID('Item') };
    push @{ $self->{'Items'} }, \%hColumns;

    return;
}

1;
