use Test::More tests => 18;
use strict;

use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::MailType::API::Object::MailType;
use DE_EPAGES::Object::API::Factory qw (
    LoadClassByAlias
    LoadRootObject
    LoadObjectByPath
    ExistsObjectByPath
);
use DE_EPAGES::Object::API::Language qw ( GetPKeyLanguageByCode );
use DE_EPAGES::Product::API::Product qw ( UpdateAvailability );
use DE_EPAGES::Product::API::Table::ProductAvailabilityEmail qw (
    GetInfoProductAvailabilityEmailsByLocaleID
    GetInfoProductAvailabilityEmailsByProductID
    InsertProductAvailabilityEmail
);
use DE_EPAGES::ShopMailType::API::ShopMailType qw (GetShopMailType);

my $ROOTPATH = '/Test';

#========================================================================================
# function     TestLocaleOfProductAvailabilityEmail
#----------------------------------------------------------------------------------------
# decription   Test checks that product email-notifications are sent in the shop
#               language/locale, in which they were originally registered --- independent
#               of the removal of this specific locale.
#========================================================================================
sub TestLocaleOfProductAvailabilityEmail {  ## no critic qw(DE_EPAGES::Style::RequireCommentForExportedFunctions)
    # (1) prepare two test shops
    my $ShopDE = LoadObjectByPath($ROOTPATH)->child('TestShopDE');
    my $ShopGB = LoadObjectByPath($ROOTPATH)->child('TestShopGB');

    # add shop mail type 'ProductAvailibityNotification'
    my $ShopMailType = GetShopMailType($ShopDE, 'ProductAvailibityNotification');
    $ShopMailType->set({ FROM => 'm.mustermann@epages.de', IsActive => 1 }); # implicitely activates mail type
    $ShopMailType = GetShopMailType($ShopGB, 'ProductAvailibityNotification');
    $ShopMailType->set({ FROM => 'm.mustermann@epages.de', IsActive => 1 }); # implicitely activates mail type

    my $DefaultShopDELocale = 'de_DE';
    $ShopDE->addLocale($DefaultShopDELocale) unless $ShopDE->existsLocale($DefaultShopDELocale);
    $ShopGB->addLocale($DefaultShopDELocale) unless $ShopGB->existsLocale($DefaultShopDELocale);

    # set default locale id for TestShopDE to de_DE
    $ShopDE->set({ LocaleID => $DefaultShopDELocale });
    is( $ShopDE->get('LocaleID'), $DefaultShopDELocale,
        "default LocaleID set to $DefaultShopDELocale for shop " . $ShopDE->alias );

    # set default locale id for TestShopGB to en_GB
    $ShopGB->set({ LocaleID => $DefaultShopDELocale });
    is( $ShopGB->get('LocaleID'), $DefaultShopDELocale,
        "default LocaleID set to $DefaultShopDELocale for shop " . $ShopGB->alias );


    # (2a) register email notification on product 'MyProduct1' (locale: de_DE)
    my $ProductDE = $ShopDE->folder('Products')->child('MyProduct1');
    my $Email = 'm.mustermann@epages.de';
    my %Params = (
        ProductID => $ProductDE->id,
        EMail     => $Email,
        LocaleID  => $DefaultShopDELocale,
    );
    InsertProductAvailabilityEmail(\%Params);

    my $aEmailNotifications = GetInfoProductAvailabilityEmailsByProductID($ProductDE->id);
    is( scalar @{ $aEmailNotifications }, 1,
        'registered a new email notification for product ' . $ProductDE->alias );
    my $hInfoEmail = $aEmailNotifications->[0];
    is( $hInfoEmail->{EMail}, $Email,
        'verified email (' . $ProductDE->alias . ')' );
    is( $hInfoEmail->{ProductID}, $ProductDE->id,
        'verified product id (' . $ProductDE->alias . ')' );
    is( $hInfoEmail->{LocaleID}, $DefaultShopDELocale,
        'verified locale id (' . $ProductDE->alias . ')' );

    # (2b) register email notification on product 'MyProduct2' (locale: de_DE)
    my $ProductGB = $ShopGB->folder('Products')->child('MyProduct2');
    %Params = (
        ProductID => $ProductGB->id,
        EMail     => $Email,
        LocaleID  => $DefaultShopDELocale,
    );
    InsertProductAvailabilityEmail(\%Params);

    $aEmailNotifications = GetInfoProductAvailabilityEmailsByProductID($ProductGB->id);
    is( scalar @{ $aEmailNotifications }, 1,
        'registered a new email notification for product ' . $ProductGB->alias );
    $hInfoEmail = $aEmailNotifications->[0];
    is( $hInfoEmail->{EMail}, $Email,
        'verified email (' . $ProductGB->alias . ')' );
    is( $hInfoEmail->{ProductID}, $ProductGB->id,
        'verified product id (' . $ProductGB->alias . ')' );
    is( $hInfoEmail->{LocaleID}, $DefaultShopDELocale,
        'verified locale id (' . $ProductGB->alias . ')' );


    # (3) remove locale de_DE from TestShopGB
    my $DefaultShopGBLocale = 'en_GB';
    $ShopGB->set({ LocaleID => $DefaultShopGBLocale });
    is( $ShopGB->get('LocaleID'), $DefaultShopGBLocale,
        "updated default locale id to $DefaultShopGBLocale for shop " . $ShopGB->alias );
    $ShopGB->removeLocale($DefaultShopDELocale);
    ok( !$ShopGB->existsLocale($DefaultShopDELocale),
        "removed locale $DefaultShopDELocale from shop " . $ShopGB->alias );


    # (4) set MyProduct1 and MyProduct2 to 'on stock', and notify customers
    $ProductDE->set({
        IsAvailable => 1,
        IsVisible   => 1,
        StockLevel  => 5,
    });
    $ProductGB->set({
        IsAvailable => 1,
        IsVisible   => 1,
        StockLevel  => 5,
    });


    # (5) update availability
    {
        local *DE_EPAGES::MailType::API::Object::MailType::send = sub {
            my $self = shift;
            my ($Object, $hMailParams, $hRegionalParams, $hVars ) = @_;

            # check old/new locales of emails
            if ($Object->id == $ProductDE->id) {
                is( $hRegionalParams->{Locale}, $DefaultShopDELocale,
                    "mail sent still with locale $DefaultShopDELocale" );
            } elsif ($Object->id == $ProductGB->id) {
                is( $hRegionalParams->{Locale}, $DefaultShopGBLocale,
                    "mail sent with new default locale $DefaultShopGBLocale" );
            }
        };
        UpdateAvailability();
    }

    return;
}

#========================================================================================
# function     TestOnDeleteLocale
# state        private
#----------------------------------------------------------------------------------------
# description  Test checks that product email notifications get deleted if the site
#               locale, by which they are registered, is deleted.
#               (cf. ProductAvailabilityEmail::OnDeleteLocale)
#========================================================================================
sub TestOnDeleteLocale {  ## no critic qw(DE_EPAGES::Style::RequireCommentForExportedFunctions)
    my $ShopDE = LoadObjectByPath($ROOTPATH)->child('TestShopDE');

    # (1) insert a new locale having the id 'xx_XX'
    my $SiteLocale = $ShopDE->siteLocale($ShopDE->get('LocaleID'));
    my $Class = LoadClassByAlias('Locale');
    my $Locale = $Class->insertObject({
        CountryID  => $SiteLocale->get('Country')->{'CountryID'},
        CurrencyID => $SiteLocale->get('Currency')->{'CurrencyID'},
        IsUsed     => 1,
        LanguageID => $SiteLocale->get('Language')->{'LanguageID'},
        LocaleID   => 'xx_XX',
    });
    ok( $ShopDE->existsSiteLocale($Locale->id), 'locale xx_XX created' );


    # (2) add another product notification with locale 'xx_XX'
    my $ProductDE = $ShopDE->folder('Products')->child('MyProduct1');
    my %Params = (
        ProductID => $ProductDE->id,
        EMail     => 'm.mustermann@epages.com',
        LocaleID  => 'xx_XX',
    );
    InsertProductAvailabilityEmail(\%Params);


    # (3) since locale 'xx_XX' is not used as default in any shop, we can just delete
    #     the locale, again.
    $Locale->delete;


    # (4) send notifications (includes implicit deletion in the database)
    {
        local *DE_EPAGES::MailType::API::Object::MailType::send = sub {
            #my $self = shift;
            #my ($Object, $hMailParams, $hRegionalParams, $hVars ) = @_;
            return;
        };
        UpdateAvailability();
    }


    # (5) verify that notification with the deleted locale 'xx_XX' are removed from db
    my $aEmails = GetInfoProductAvailabilityEmailsByLocaleID($Locale->id);
    is( scalar @{$aEmails}, 0, 'no more notifications stored for locale ' . $Locale->id );

    return;
}

sub Cleanup {
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    my $Class = LoadClassByAlias('Locale');
    if ($Class->existsObject('xx_XX')) {
        my $Locale = $Class->loadObject('xx_XX');
        $Locale->delete;
    }

    return 1;
}

sub Init {
    Cleanup();

    my $Folder = LoadRootObject()->insertChild('Object', { Alias => 'Test' });
    my $ShopDE = $Folder->insertChild('Shop', { Alias => 'TestShopDE' });
    my $ShopGB = $Folder->insertChild('Shop', { Alias => 'TestShopGB' });

    my $LanguageID = GetPKeyLanguageByCode('de');
    $ShopDE->featureMaxValue('Languages', 2);
    $ShopGB->featureMaxValue('Languages', 2);
    $ShopDE->addLanguage($LanguageID);
    $ShopGB->addLanguage($LanguageID);

    $ShopDE->featureMaxValue('Products', 1);
    my $ProductFolder = $ShopDE->get('ProductFolder');
    $ProductFolder->insertChild('Product', { Alias => 'MyProduct1' });
    $ShopGB->featureMaxValue('Products', 1);
    $ProductFolder = $ShopGB->get('ProductFolder');
    $ProductFolder->insertChild('Product', { Alias => 'MyProduct2' });

    return 1;
}

RunScript(
    Message => 'long',
    Sub     => sub {
        RunOnStore(
            Store => GetTestSite(),
            Sub   => sub {
                ok( Init(), 'initialization done' );

                TestLocaleOfProductAvailabilityEmail();
                TestOnDeleteLocale();

                ok( Cleanup(), 'cleanup done' );
            }
        );
    },
);