#========================================================================================
# §package      DE_EPAGES::RemoteSearch::t::SearchClient
# §state        private
#========================================================================================
package DE_EPAGES::RemoteSearch::t::SearchClient;
use base Test::Class;

use strict;
use warnings;
use DE_EPAGES::Core::API::Object::BaseObject;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::RemoteSearch::API::Object::HTTPClient;
use DE_EPAGES::RemoteSearch::API::Object::SearchClient;
use DE_EPAGES::RemoteSearch::API::Object::SearchParamBuilder;
use HTTP::Response;
use HTTP::Headers;

use HTTP::Status qw ( :constants :is status_message );
use JSON;
use Test::MockObject;
use Test::More;
use utf8;

my $DEFAULT_RESPONSE = '{"result": [1234, 4711, 7, 23, 127]}';
my $UMLAUT_RESPONSE  = '{"query":"mäuse","result":[]}';

use constant PRICE_LOW => 11.1;
use constant PRICE_HIGH => 22.22;

sub testSimpleSearch : Test(no_plan) {
    my $self = shift;
    my $cMethod = sub {
                    shift;
                    my ( $URL, $hHeader, $Content ) = @_;
                    is( $URL->abs, 'http://localhost:8088/rs-test/product/Store/GUID/search'
                                 , 'post path for SimpleSearch' );
                    _ReturnOk($DEFAULT_RESPONSE);
                  };

    my $Client = _MockSearchClient('post' => $cMethod);

    is($Client->searchUrl(),q#http://localhost:8088/rs-test/product/Store/GUID/search#, 'getter searchUrl');

    my $Builder = DE_EPAGES::RemoteSearch::API::Object::SearchParamBuilder->new();
    $Builder->addFilterValues( 'IsNew', [1] )
            ->addFilterRange( 'EUR', [PRICE_LOW, PRICE_HIGH] )
            ->query('test')
            ->resultsPerPage(5)
            ->sortBy('relevance')
            ->lang('de');

    my $Response = $Client->search( $Builder->params() );
    isa_ok( $Response, 'HTTP::Response' );

    is( $Response->decoded_content, $DEFAULT_RESPONSE, 'content ok' );
}

sub testSearchAsHash : Test(no_plan) {
    my $self   = shift;
    my $Client = _MockSearchClient('post' => sub { _ReturnOk($DEFAULT_RESPONSE);});

    my $Builder = DE_EPAGES::RemoteSearch::API::Object::SearchParamBuilder->new();
    $Builder->addFilterValues( 'IsNew', [1] )
            ->addFilterRange( 'EUR', [PRICE_LOW, PRICE_HIGH] )
            ->query('test')
            ->resultsPerPage(5)
            ->sortBy('relevance')
            ->lang('de');

    my $Response = $Client->searchAsHash( $Builder->params() );
    is( $Response->{result}->[0], 1234, 'check hash contents' );## no critic (ValuesAndExpressions::ProhibitMagicNumbers)
}

sub testSearchWithUmlauts : Test(no_plan) {
    my $self   = shift;
    my $Client = _MockSearchClient('post' => sub { _ReturnOk($UMLAUT_RESPONSE);});

    my $Builder = DE_EPAGES::RemoteSearch::API::Object::SearchParamBuilder->new();
    $Builder->addFilterValues( 'IsNew', [1] )
            ->addFilterRange( 'EUR', [PRICE_LOW, PRICE_HIGH] )
            ->query('mäuse')
            ->resultsPerPage(5)
            ->sortBy('relevance')
            ->lang('de');

    my $Response = $Client->searchAsHash( $Builder->params() );
    is( $Response->{query}, 'mäuse', 'Umlaut ok.' );
}

# Factory for SearchClient with fake HTTPClient and custom get/post method 
sub _MockSearchClient {
    my ($Name, $cMethod) = @_;

    my $mockRemote = Test::MockObject->new();

    $mockRemote->set_isa('DE_EPAGES::RemoteSearch::API::Object::HTTPClient');
    $mockRemote->fake_module( 'DE_EPAGES::RemoteSearch::API::Object::HTTPClient', %{{$Name => $cMethod}} );
    $mockRemote->mock( $Name, $cMethod );

    return DE_EPAGES::RemoteSearch::API::Object::SearchClient->new(
        'BaseURL'    => 'http://localhost:8088/rs-test/',
        'Datasource' => 'Store',
        'Resource'   => 'GUID',
        'HttpClient' => $mockRemote,
        );

}

sub _ReturnOk {
    my ($Content) = @_;
    utf8::encode($Content);
    return HTTP::Response->new( HTTP_OK, status_message(HTTP_OK), HTTP::Headers->new('Content-Type' => 'application/json; charset=utf-8'), $Content );
}

####### Main ############
RunScript(
    Sub => sub {
        Test::Class->runtests;
      }
);
